/**
 * XmppSession
 * @author Skitsanos.com
 * @version 1.2(XMPP3)
 */ 
package core
{
	import com.skitsanos.Grafeio.GrafeioEnvironment;
	import com.skitsanos.Grafeio.events.EventHub;
	import com.skitsanos.Grafeio.events.GrafeioApplicationEvent;
	
	import commands.JoinConferenceCmd;
	
	import flash.events.Event;
	import flash.events.TimerEvent;
	import flash.system.Security;
	import flash.utils.Dictionary;
	import flash.utils.Timer;
	
	import org.igniterealtime.xiff.conference.InviteListener;
	import org.igniterealtime.xiff.core.EscapedJID;
	import org.igniterealtime.xiff.core.XMPPConnection;
	import org.igniterealtime.xiff.data.IQ;
	import org.igniterealtime.xiff.data.Message;
	import org.igniterealtime.xiff.data.Presence;
	import org.igniterealtime.xiff.data.muc.MUC;
	import org.igniterealtime.xiff.data.muc.MUCItem;
	import org.igniterealtime.xiff.data.muc.MUCUserExtension;
	import org.igniterealtime.xiff.data.search.SearchExtension;
	import org.igniterealtime.xiff.events.ConnectionSuccessEvent;
	import org.igniterealtime.xiff.events.DisconnectionEvent;
	import org.igniterealtime.xiff.events.IncomingDataEvent;
	import org.igniterealtime.xiff.events.InviteEvent;
	import org.igniterealtime.xiff.events.LoginEvent;
	import org.igniterealtime.xiff.events.MessageEvent;
	import org.igniterealtime.xiff.events.OutgoingDataEvent;
	import org.igniterealtime.xiff.events.PresenceEvent;
	import org.igniterealtime.xiff.events.RegistrationFieldsEvent;
	import org.igniterealtime.xiff.events.RegistrationSuccessEvent;
	import org.igniterealtime.xiff.events.RosterEvent;
	import org.igniterealtime.xiff.events.XIFFErrorEvent;
	import org.igniterealtime.xiff.im.Roster;
	import org.igniterealtime.xiff.privatedata.PrivateDataManager;
	
	public class XmppSession
	{
		private var loginJID:String;
		private var loginPwd:String;
		private var loginServer:String;
		
		private var _conn:XMPPConnection;
        private var _privData:PrivateDataManager;  // for storage private data
        
        private var _roster:Roster;
        private var _currentUserJID:String;
        private var _regdata:Object;
        private var _messageDict:Dictionary;
        private var _doRegister:Boolean;
        private var _inviteListener:InviteListener;
        private var _addBotFlag:Boolean = false;
        
        //to keep alive the connection to XMPP server we need to ping server regulary
        private var keepAlive:Timer;
        private var _lastSent:int = 0;
		
		public function XmppSession(loginJID:String = null, loginPwd:String = null, loginServer:String = null, anonymous:Boolean = false)
		{	
			SearchExtension.enable();
			MUC.enable();
			
			_conn = new XMPPConnection();
            _conn.resource = "Grafeio";
            _conn.useAnonymousLogin = anonymous;
            
            _inviteListener = new InviteListener();
            _inviteListener.setConnection(_conn);
			
			this.loginJID = loginJID;
			this.loginPwd = loginPwd;
			this.loginServer = loginServer;
		
			initObjects();
			
            setupConnectionListeners();
            setupRosterListeners();
            setupPresenceListeners();
            setupMessageListeners();
            _doRegister = false;
            
            keepAlive = new Timer(15000);
            keepAlive.addEventListener(TimerEvent.TIMER, onKeepAliveLoop);
		}
			
		private function initObjects() : void
        {
            _messageDict = new Dictionary();
            MUC.enable();
        }
			
		private function setupConnectionListeners() : void
        {
            _inviteListener.addEventListener(InviteEvent.INVITED, handleInvite);
            _conn.addEventListener(ConnectionSuccessEvent.CONNECT_SUCCESS,onConnectSuccess);
            _conn.addEventListener(XIFFErrorEvent.XIFF_ERROR, onError);
            _conn.addEventListener(OutgoingDataEvent.OUTGOING_DATA,onOutgoingData)
            _conn.addEventListener(IncomingDataEvent.INCOMING_DATA,onIncomingData);
                
            _conn.addEventListener(DisconnectionEvent.DISCONNECT,onDisconnect);
            _conn.addEventListener(LoginEvent.LOGIN,onLogin);
            
            _conn.addEventListener(PresenceEvent.PRESENCE, _presence_chaged);
                                   
            _conn.addEventListener(RegistrationFieldsEvent.REG_FIELDS, onRegistrationFields);
            _conn.addEventListener(RegistrationSuccessEvent.REGISTRATION_SUCCESS, onRegistrationSuccess);                
         }
            
         private function setupPresenceListeners() : void
         {
             _conn.addEventListener(PresenceEvent.PRESENCE,presenceHandler);
         }
            
         private function setupMessageListeners() : void
         {
              _conn.addEventListener(MessageEvent.MESSAGE,messageHandler);
         }
            
         private function setupRosterListeners() : void
         {
             _roster = new Roster(_conn);
             _roster.addEventListener(RosterEvent.SUBSCRIPTION_DENIAL, rosterHandler);
             _roster.addEventListener(RosterEvent.SUBSCRIPTION_REQUEST, rosterHandler);
             _roster.addEventListener(RosterEvent.SUBSCRIPTION_REVOCATION, rosterHandler);
             _roster.addEventListener(RosterEvent.USER_AVAILABLE, rosterHandler);
             _roster.addEventListener(RosterEvent.USER_UNAVAILABLE, rosterHandler);               
         }
         
         private function handleInvite(e:InviteEvent) : void
         {
                trace('handle Invitation to room: ' + e.room.roomName);
         }
           
         private function inviteAcceptHandler(e:Event) : void
         {
   	     }
                                          
		 private function sendMessage(e:MessageEvent) : void
         {
         	var msg:Message = e.data as Message;
            _conn.send(msg);
         }
         
         public function register(loginMail:String = null) : void
         {
            _doRegister = true;
                
            _conn.username = null;
            _conn.password = null;    
                
            _conn.server = loginServer;
            _conn.connect();
            
            _regdata = {};
            _regdata.username = loginJID;  // required 
            _regdata.password = loginPwd; // required
            if(loginMail != null)
            {
           		_regdata.email = loginMail; // optional
           		trace(loginMail);
           	}
         }
         
         private function onRegistrationSuccess(e:RegistrationSuccessEvent) : void
         {
             //drop old connection 
             _conn.disconnect();
             connect();
             // alert the user
             //trace("Registration was successful. You will now be logged in. You may want to add some contacts at this point.");
             
             GrafeioEnvironment.username = _conn.username;

             // Add the bot for testing
             _addBotFlag = true;
         }
         
         private function onRegistrationFields(e:RegistrationFieldsEvent) : void
         {
         }         
            
         private function rosterHandler(e:RosterEvent):void
         {
             switch (e.type){
             	case RosterEvent.SUBSCRIPTION_REQUEST :
                        /* we will check to see if this user is already in roster.
                            if so we will auto accept */
                     if (_roster.getPresence(e.jid) != null)
                     {
                            _roster.grantSubscription(e.jid, true);
                     }
                     else
                     {
                     }
                    break;
                    
                case RosterEvent.USER_UNAVAILABLE : 
                     trace (e.jid.bareJID + " is Unavailable (RosterEvent)");
                     break;
                     
                case RosterEvent.USER_AVAILABLE : 
                     trace (e.jid.bareJID + " is Available (RosterEvent)");
                     break;
                     
                case RosterEvent.SUBSCRIPTION_DENIAL : 
                     trace (e.jid.bareJID + " denied your request (RosterEvent)");
                     break;
                     
                case RosterEvent.SUBSCRIPTION_REVOCATION : 
                     trace (e.jid.bareJID + " revoked your presence (RosterEvent)");
                     break;
                     
                 default :
                     // do nothing... not recognized
       	     }
         }
            
            private function presenceHandler(e:PresenceEvent):void
            {
                for each(var ob:Object in e.data)
                {
              		 var p:Presence = ob as Presence;
                     var jid:String = p.from.toString(); // We will use JID as our key
                  switch (p.type){
                    case Presence.ERROR_TYPE :
                        var error:XIFFErrorEvent = new XIFFErrorEvent();
                        error.errorCode = p.errorCode;
                        error.errorCondition = p.errorCondition;
                        error.errorMessage = p.errorMessage;
                        error.errorType = p.errorType;
                        onError(error);
                        break;
                    case Presence.SUBSCRIBED_TYPE :
                        trace (jid + " Presence.AVAILABLE_TYPE (PresenceEvent)");
                        break;
                    case Presence.UNAVAILABLE_TYPE :
                        trace (jid + " Presence.UNAVAILABLE_TYPE (PresenceEvent)");
                        break;
                    case Presence.UNSUBSCRIBE_TYPE :
                        trace (jid + " Presence.UNSUBSCRIBE_TYPE (PresenceEvent)");    
                        break;
                    default : 
                        break;
                 }
               }
              
            }
            
            private function messageHandler(e:MessageEvent):Boolean
			{
				var m:Message = e.data as Message;
				switch (m.type){
					case Message.CHAT_TYPE :
						trace("chat");
						break;
						
					case Message.ERROR_TYPE :
						var error:XIFFErrorEvent = new XIFFErrorEvent();
						error.errorCode = m.errorCode;
						error.errorCondition = m.errorCondition;
						error.errorMessage = m.errorMessage;
						error.errorType = m.errorType;
						onError(error);
						break;
						
					case Message.GROUPCHAT_TYPE:
						m.getAllExtensions()
						break;
					
					default : 
						//trace("Message not handled.");
				}
				return true;
			}
            
            private function onOutgoingData(e:OutgoingDataEvent):void
            {
                //trace("onOutgoingData: " + e.data); 
            }
            private function onIncomingData(e:IncomingDataEvent):void
            {
               // trace("onIncomingData: " + e.data.toString()); 
                /*
                switch (e.data.attributes["type"])
                {
                	case "groupchat":
                		trace("--chat room:");
                		break;
                }
                */
            }
              
            public function connect() : void
            {
            	Security.loadPolicyFile(loginServer + "/crossdomain.xml");
            	
            	_currentUserJID = loginJID;
            	_doRegister = false;
            	
                _conn.username = loginJID;                
                _conn.password = loginPwd;
                _conn.server = loginServer;
                
                if (_conn.isLoggedIn())
                {
                	_conn.disconnect();
                }
                _conn.connect();
                
                keepAlive.start();
            }
            
            public function disconnect() : void
            {
                _conn.disconnect();
            }
             
            private function onLogin(e:LoginEvent):void
            {				
				GrafeioEnvironment.username = _conn.username;
						
				if (_conn.useAnonymousLogin == false)
				{					
					GrafeioEnvironment.fso.account = _conn.username;
					EventHub.getInstance().broadcast(new Event(EventsEnum.XMPP_EVENT_USERLOGIN));					
				}
				else
				{
					GrafeioEnvironment.fso.account = "anonymous"; //anoinimous users share the same disk space
					EventHub.getInstance().broadcast(new Event(EventsEnum.XMPP_EVENT_ANONYMOUSLOGIN));					
				}
				
				enterConference();    
            }
            
            private function onConnectSuccess( e:ConnectionSuccessEvent):void
            {
                if (_doRegister)
                {
                    EventHub.getInstance().broadcast(new Event(EventsEnum.XMPP_EVENT_CONNECTED));
                    _conn.sendRegistrationFields( _regdata, null );
                }
            }
            
            private function onDisconnect(e:DisconnectionEvent):void
            {
                trace(">>> DISCONNECTED")
            }
            
            private function onError(e:XIFFErrorEvent):void
            {
                if (e.errorCode == 401)
                {
                	trace(">>> XIFFErrorEvent:" +e.errorMessage);
                	
                }
                else
                {
                	var error:Object = {errorCode:e.errorCode,errorCondition:e.errorCondition,errorMessage:e.errorMessage,errorType:e.errorType};
                	trace("onError: " + "type:" + e.errorType + "  message:" + e.errorMessage); 
                }                
            }
            
        private function enterConference() : void
		{
			var cmdJoinConf:JoinConferenceCmd = new JoinConferenceCmd(_conn,_roster);
			cmdJoinConf.execute();
		}
		/**
		 * Handles user presence status change, 
		 * tracks when user enters and leaves the room and fires an event on these actions;
		 * presence packet sent by "from" and says what status "to" has in each presence change action;
		 */ 
		private function _presence_chaged(e:PresenceEvent):void
		{
            for (var i:uint = 0; i < e.data.length; ++i)
            {
            	var presence:Presence = e.data[i] as Presence;
            	                
                for each(var ext:MUCUserExtension in presence.getAllExtensions())
                {
                	for each(var item:MUCItem in ext.getAllItems())
                	{
                		switch (item.role)
                		{
                			case "participant":
								//user is the participant in the room
								EventHub.getInstance().broadcast(new GrafeioApplicationEvent(EventsEnum.HOOD_EVENT_USER_ONLINE, item.jid,true));
								break;
							
							case "none":
								//user left the room
								EventHub.getInstance().broadcast(new GrafeioApplicationEvent(EventsEnum.HOOD_EVENT_USER_OFFLINE, item.jid,true));
								break;
							
							default:
								break;
                		}
                	}
                }
            }          
		}
		
		/**
		 * To keep session connected we need to send keep-alive packet 
		 */ 
		private function onKeepAliveLoop(evt:TimerEvent):void
        {        
           if(new Date().getTime() - _lastSent > 15000)
				_conn.sendKeepAlive();
        }
		/**
		 * Search for user via XMPP
		 */ 
		public function searchUsers(username:String):void
		{
			var iqSearch:IQ = new IQ(new EscapedJID(username + "@" + _conn.server, false), IQ.GET_TYPE);
			iqSearch.addExtension(new SearchExtension());
			
			_conn.send(iqSearch);
		}
		private function getSearchFields(resultIQ:IQ):void
		{
			var ext:Array = resultIQ.getAllExtensions();
			trace(ext.length);
		}
	}
}